@@ -18,7 +18,7 @@ use std::io::{self, BufReader, BufWriter, Error, ErrorKind};
 use std::mem;
 use std::sync::Arc;
 
-use std::thread;
+use std::thread::{self, JoinHandle};
 use std::time::{Duration, Instant};
 
 use crossbeam_channel as channel;
@@ -38,9 +38,9 @@ const BUF_CAPACITY: usize = 32 * 1024;
 ///     first and released after when both are used.
 ///     Failure to follow this strict rule WILL create
 ///     a deadlock!
-struct State {
+pub(crate) struct State {
     write: Mutex<WriteState>,
-    read: Mutex<ReadState>,
+    pub(crate) read: Mutex<ReadState>,
     meta: Mutex<MetaState>,
 }
 
@@ -69,9 +69,9 @@ struct WriteState {
     next_sid: u64,
 }
 
-struct ReadState {
+pub(crate) struct ReadState {
     /// Current subscriptions.
-    subscriptions: HashMap<u64, Subscription>,
+    pub(crate) subscriptions: HashMap<u64, Subscription>,
 
     /// Expected pongs and their notification channels.
     pongs: VecDeque<channel::Sender<()>>,
@@ -87,18 +87,20 @@ struct ReadState {
 pub(crate) type Preprocessor = Box<dyn Fn(u64, &Message) -> bool + Send + Sync>;
 
 /// A registered subscription.
-struct Subscription {
+pub(crate) struct Subscription {
     subject: String,
     queue_group: Option<String>,
     messages: channel::Sender<Message>,
     preprocess: Preprocessor,
+    pub(crate) pending_messages_limit: Option<usize>,
+    pub(crate) dropped_messages: usize,
 }
 
 /// A NATS client.
 #[derive(Clone)]
 pub struct Client {
     /// Shared client state.
-    state: Arc<State>,
+    pub(crate) state: Arc<State>,
 
     /// Server info provided by the last INFO message.
     pub(crate) server_info: Arc<Mutex<ServerInfo>>,
@@ -108,6 +110,12 @@ pub struct Client {
 
     /// The options that this `Client` was created using.
     pub(crate) options: Arc<Options>,
+
+    /// handler of client thread.
+    pub(crate) client_thread: Arc<Mutex<Option<JoinHandle<()>>>>,
+
+    /// handler of flush thread.
+    pub(crate) flush_thread: Arc<Mutex<Option<JoinHandle<()>>>>,
 }
 
 impl Client {
@@ -142,6 +150,8 @@ impl Client {
             server_info: Arc::new(Mutex::new(ServerInfo::default())),
             shutdown: Arc::new(Mutex::new(false)),
             options: Arc::new(options),
+            client_thread: Arc::new(Mutex::new(None)),
+            flush_thread: Arc::new(Mutex::new(None)),
         };
 
         let options = client.options.clone();
@@ -155,7 +165,7 @@ impl Client {
         //   broken.
         // - Reading messages from the server and processing them.
         // - Forwarding MSG operations to subscribers.
-        thread::spawn({
+        let handle = thread::spawn({
             let client = client.clone();
             move || {
                 let res = client.run(connector);
@@ -175,6 +185,8 @@ impl Client {
             }
         });
 
+        *client.client_thread.lock() = Some(handle);
+
         channel::select! {
             recv(run_receiver) -> res => {
                 res.expect("client thread has panicked")?;
@@ -184,7 +196,7 @@ impl Client {
         }
 
         // Spawn a thread that periodically flushes buffered messages.
-        thread::spawn({
+        let handle = thread::spawn({
             let client = client.clone();
             move || {
                 // Track last flush/write time.
@@ -199,6 +211,10 @@ impl Client {
 
                 // Wait until at least one message is buffered.
                 loop {
+                    // if client is shutting down, stop periodic flushses.
+                    if client.check_shutdown().is_err() {
+                        break;
+                    }
                     match flush_wanted.recv_timeout(PING_INTERVAL) {
                         Ok(_) => {
                             let since = last.elapsed();
@@ -265,6 +281,7 @@ impl Client {
             }
         });
 
+        *client.flush_thread.lock() = Some(handle);
         Ok(client)
     }
 
@@ -326,6 +343,8 @@ impl Client {
         let mut write = self.state.write.lock();
         let mut read = self.state.read.lock();
 
+        write.flush_kicker.try_send(()).ok();
+
         // Initiate shutdown process.
         if self.shutdown() {
             // Clear all subscriptions.
@@ -341,16 +360,23 @@ impl Client {
             read.subscriptions.clear();
 
             // Flush the writer in case there are buffered messages.
-            if let Some(writer) = write.writer.as_mut() {
+            if let Some(mut writer) = write.writer.as_mut() {
+                // TODO: for some reason sometimes Push Consumer Subscription cause
+                // `close()` to hang. Sending ping unblocks read_line. Not worth investigating further
+                // this edge case as async client will not have this issue.
+                proto::encode(&mut writer, ClientOp::Ping).ok();
                 writer.flush().ok();
             }
 
             // Wake up all pending flushes.
             read.pongs.clear();
-
             // NB see locking protocol for state.write and state.read
             drop(read);
             drop(write);
+
+            // wait for the threads.
+            self.client_thread.lock().take().map(JoinHandle::join);
+            self.flush_thread.lock().take().map(JoinHandle::join);
         }
     }
 
@@ -419,6 +445,8 @@ impl Client {
                 queue_group: queue_group.map(ToString::to_string),
                 messages: sender,
                 preprocess: message_processor,
+                pending_messages_limit: None,
+                dropped_messages: 0,
             },
         );
 
@@ -859,7 +887,7 @@ impl Client {
                         continue;
                     }
 
-                    let read = self.state.read.lock();
+                    let mut read = self.state.read.lock();
 
                     // Send the message to matching subscription.
                     if let Some(subscription) = read.subscriptions.get(&sid) {
@@ -879,6 +907,26 @@ impl Client {
                             continue;
                         }
 
+                        //check if subscription has set limits for slow consumers
+                        if let Some(pending_messages_limit) = subscription.pending_messages_limit {
+                            if pending_messages_limit <= subscription.messages.len() {
+                                connector.get_options().error_callback.call(
+                                    self,
+                                    io::Error::new(
+                                        ErrorKind::Other,
+                                        format!(
+                                            "slow consumer detected for subscription on subject {}. dropping messages",
+                                            subscription.subject
+                                        ),
+                                    ),
+                                );
+                                read.subscriptions
+                                    .entry(sid)
+                                    .and_modify(|sub| sub.dropped_messages += 1);
+                                continue;
+                            }
+                        }
+
                         // Send a message or drop it if the channel is
                         // disconnected or full.
                         subscription.messages.send(msg).unwrap();
@@ -916,6 +964,23 @@ impl Client {
                             continue;
                         }
 
+                        //check if subscription has set limits for slow consumers
+                        if let Some(pending_messages_limit) = subscription.pending_messages_limit {
+                            if pending_messages_limit <= subscription.messages.len() {
+                                connector.get_options().error_callback.call(
+                                    self,
+                                    io::Error::new(
+                                        ErrorKind::Other,
+                                        format!(
+                                            "slow consumer detected for subscription on subject {}. dropping messages",
+                                            subscription.subject
+                                        ),
+                                    ),
+                                );
+                                println!("reached slow consumers");
+                                continue;
+                            }
+                        }
                         // Send a message or drop it if the channel is
                         // disconnected or full.
                         subscription.messages.send(msg).unwrap();
