@@ -95,29 +95,36 @@
 //! internally managed consumer resource that gets destroyed when the subscription is dropped.
 //!
 use std::{
-    collections::{HashSet, VecDeque},
+    collections::VecDeque,
     convert::TryFrom,
     error, fmt,
     fmt::Debug,
     io::{self, ErrorKind},
+    time::Duration,
 };
 
 use parking_lot::Mutex;
 use std::sync::atomic::{AtomicU64, Ordering};
 use std::sync::Arc;
 use std::thread;
-use std::time::Duration;
 
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
 use serde_repr::{Deserialize_repr, Serialize_repr};
 
 const ORDERED_IDLE_HEARTBEAT: Duration = Duration::from_nanos(5_000_000_000);
 
-mod pull_subscription;
-mod push_subscription;
+/// Pull subscriptions
+pub mod pull_subscription;
+
+/// Push subscriptions
+pub mod push_subscription;
+
 mod types;
 
-pub use push_subscription::PushSubscription;
+// We use a fully qualified crate path so these are documented as re-exports.
+pub use crate::jetstream::pull_subscription::PullSubscription;
+pub use crate::jetstream::push_subscription::PushSubscription;
+
 pub use types::*;
 
 use crate::{
@@ -125,8 +132,6 @@ use crate::{
     Connection, Message,
 };
 
-use self::pull_subscription::PullSubscription;
-
 /// `JetStream` options
 #[derive(Clone)]
 pub struct JetStreamOptions {
@@ -608,48 +613,23 @@ impl JetStream {
             let mut headers = maybe_headers.map_or_else(HeaderMap::default, HeaderMap::clone);
 
             if let Some(v) = options.id.as_ref() {
-                let entry = headers
-                    .inner
-                    .entry(header::NATS_MSG_ID.to_string())
-                    .or_insert_with(HashSet::default);
-
-                entry.insert(v.to_string());
+                headers.insert(header::NATS_MSG_ID, v.to_string());
             }
 
             if let Some(v) = options.expected_last_msg_id.as_ref() {
-                let entry = headers
-                    .inner
-                    .entry(header::NATS_EXPECTED_LAST_MSG_ID.to_string())
-                    .or_insert_with(HashSet::default);
-
-                entry.insert(v.to_string());
+                headers.insert(header::NATS_EXPECTED_LAST_MSG_ID, v.to_string());
             }
 
             if let Some(v) = options.expected_stream.as_ref() {
-                let entry = headers
-                    .inner
-                    .entry(header::NATS_EXPECTED_STREAM.to_string())
-                    .or_insert_with(HashSet::default);
-
-                entry.insert(v.to_string());
+                headers.insert(header::NATS_EXPECTED_STREAM, v.to_string());
             }
 
             if let Some(v) = options.expected_last_sequence.as_ref() {
-                let entry = headers
-                    .inner
-                    .entry(header::NATS_EXPECTED_LAST_SEQUENCE.to_string())
-                    .or_insert_with(HashSet::default);
-
-                entry.insert(v.to_string());
+                headers.insert(header::NATS_EXPECTED_LAST_SEQUENCE, v.to_string());
             }
 
             if let Some(v) = options.expected_last_subject_sequence.as_ref() {
-                let entry = headers
-                    .inner
-                    .entry(header::NATS_EXPECTED_LAST_SUBJECT_SEQUENCE.to_string())
-                    .or_insert_with(HashSet::default);
-
-                entry.insert(v.to_string());
+                headers.insert(header::NATS_EXPECTED_LAST_SUBJECT_SEQUENCE, v.to_string());
             }
 
             Some(headers)
@@ -1330,31 +1310,26 @@ impl JetStream {
                     let maybe_consumer_stalled = message
                         .headers
                         .as_ref()
-                        .and_then(|headers| {
-                            headers
-                                .get(header::NATS_CONSUMER_STALLED)
-                                .map(|set| set.iter().cloned().next())
-                        })
-                        .flatten();
+                        .and_then(|headers| headers.get(header::NATS_CONSUMER_STALLED));
 
                     if let Some(consumer_stalled) = maybe_consumer_stalled {
                         context.connection.try_publish_with_reply_or_headers(
-                            &consumer_stalled,
+                            consumer_stalled,
                             None,
                             None,
                             b"",
                         );
                     }
 
+                    // if it is not an ordered consumer, don't handle sequence mismatch.
+                    if is_ordered {
+                        return false;
+                    }
+
                     let maybe_consumer_seq = message
                         .headers
                         .as_ref()
-                        .and_then(|headers| {
-                            headers
-                                .get(header::NATS_LAST_CONSUMER)
-                                .map(|set| set.iter().cloned().next())
-                        })
-                        .flatten();
+                        .and_then(|headers| headers.get(header::NATS_LAST_CONSUMER));
 
                     if let Some(consumer_seq) = maybe_consumer_seq {
                         let consumer_seq = consumer_seq.parse::<u64>().unwrap();
@@ -1367,6 +1342,11 @@ impl JetStream {
                     return false;
                 }
 
+                // if it is not an ordered consumer, don't handle sequence mismatch.
+                if !is_ordered {
+                    return false;
+                }
+
                 // Track messages for sequence mismatches.
                 if let Some(message_info) = message.jetstream_message_info() {
                     let mut sequence_info = sequence_pair.lock();
